use leptos::*;
use crate::app::models::Post;

#[server]
pub async fn get_posts(
    folder: String
) -> Result<Vec<Post>, ServerFnError> {
    let posts = Post::get_all(&folder)
        .map_err(|e| ServerFnError::ServerError(e.to_string()))?;

    Ok(posts)
}

#[server]
pub async fn get_post(
    folder: String,
    slug: String
) -> Result<Post, ServerFnError> {
    let posts = Post::get_all(&folder)
        .map_err(|e| ServerFnError::ServerError(e.to_string()))?;

    let post = posts.into_iter().find(|post| post.slug == slug)
        .ok_or(ServerFnError::ServerError("Post not found".to_string()))?;

    Ok(post)
}

#[component]
pub fn PostList(
    folder: String
) -> impl IntoView {
    let posts = create_resource(|| (), |_| get_posts("posts".to_string()));

    let posts_view = move || {
        posts.and_then(|posts| {
            posts.iter()
                .map(|post| view! {
                                <li>
                                    <a href={format!("/posts/{}", post.slug.clone())}>{post.metadata.title.clone()}</a>
                                </li>
                            })
                .collect_view()
        })
    };

    view! {
        <main class="m-0 p-0 bg-surface dark:bg-dark_surface text-on_surface dark:text-dark_on_surface">
            <h1>"My Great Blog"</h1>

            <Suspense fallback=move || view! { <p>"Loading posts..."</p> }>
                <ul>{posts_view}</ul>
            </Suspense>
        </main>
    }
}

#[component]
pub fn PostElement(
    folder: String,
) -> impl IntoView {
    let post = create_resource(|| (), |_| get_post("posts".to_string(), "2023-11-26_testing_layout".to_string()));

    let post_view = move || {
        post.and_then(|post| {
            view! {
                <div inner_html={post.content.clone()}></div>
            }
        })
    };

    view! {
        <main class="m-0 p-0 bg-surface dark:bg-dark_surface text-on_surface dark:text-dark_on_surface">
            <Suspense fallback=move || view! { <p>"Loading posts..."</p> }>
                {post_view}
            </Suspense>
        </main>
    }
}